"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const config_1 = require("@pnpm/config");
const plugin_commands_installation_1 = require("@pnpm/plugin-commands-installation");
const read_package_json_1 = require("@pnpm/read-package-json");
const read_project_manifest_1 = require("@pnpm/read-project-manifest");
const pick_js_1 = __importDefault(require("ramda/src/pick.js"));
const safe_execa_1 = __importDefault(require("safe-execa"));
const escape_string_regexp_1 = __importDefault(require("escape-string-regexp"));
const render_help_1 = __importDefault(require("render-help"));
exports.rcOptionsTypes = cliOptionsTypes;
function cliOptionsTypes() {
    return (0, pick_js_1.default)([], config_1.types);
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.commandNames = ['patch-commit'];
function help() {
    return (0, render_help_1.default)({
        description: 'Generate a patch out of a directory',
        descriptionLists: [],
        url: (0, cli_utils_1.docsUrl)('patch-commit'),
        usages: ['pnpm patch-commit <patchDir>'],
    });
}
exports.help = help;
async function handler(opts, params) {
    const userDir = params[0];
    const srcDir = path_1.default.join(userDir, '../source');
    const patchContent = await diffFolders(srcDir, userDir);
    const lockfileDir = opts.lockfileDir ?? opts.dir ?? process.cwd();
    const patchesDir = path_1.default.join(lockfileDir, 'patches');
    await fs_1.default.promises.mkdir(patchesDir, { recursive: true });
    const patchedPkgManifest = await (0, read_package_json_1.fromDir)(srcDir);
    const pkgNameAndVersion = `${patchedPkgManifest.name}@${patchedPkgManifest.version}`;
    const patchFileName = pkgNameAndVersion.replace('/', '__');
    await fs_1.default.promises.writeFile(path_1.default.join(patchesDir, `${patchFileName}.patch`), patchContent, 'utf8');
    let { manifest, writeProjectManifest } = await (0, read_project_manifest_1.tryReadProjectManifest)(lockfileDir);
    if (!manifest) {
        manifest = {};
    }
    if (!manifest.pnpm) {
        manifest.pnpm = {
            patchedDependencies: {},
        };
    }
    else if (!manifest.pnpm.patchedDependencies) {
        manifest.pnpm.patchedDependencies = {};
    }
    manifest.pnpm.patchedDependencies[pkgNameAndVersion] = `patches/${patchFileName}.patch`;
    await writeProjectManifest(manifest);
    return plugin_commands_installation_1.install.handler(opts);
}
exports.handler = handler;
async function diffFolders(folderA, folderB) {
    const folderAN = folderA.replace(/\\/g, '/');
    const folderBN = folderB.replace(/\\/g, '/');
    let stdout;
    let stderr;
    try {
        const result = await (0, safe_execa_1.default)('git', ['-c', 'core.safecrlf=false', 'diff', '--src-prefix=a/', '--dst-prefix=b/', '--ignore-cr-at-eol', '--full-index', '--no-index', '--text', folderAN, folderBN], {
            cwd: process.cwd(),
            env: {
                ...process.env,
                // #region Predictable output
                // These variables aim to ignore the global git config so we get predictable output
                // https://git-scm.com/docs/git#Documentation/git.txt-codeGITCONFIGNOSYSTEMcode
                GIT_CONFIG_NOSYSTEM: '1',
                HOME: '',
                XDG_CONFIG_HOME: '',
                USERPROFILE: '',
                // #endregion
            },
        });
        stdout = result.stdout;
        stderr = result.stderr;
    }
    catch (err) { // eslint-disable-line
        stdout = err.stdout;
        stderr = err.stderr;
    }
    // we cannot rely on exit code, because --no-index implies --exit-code
    // i.e. git diff will exit with 1 if there were differences
    if (stderr.length > 0)
        throw new Error(`Unable to diff directories. Make sure you have a recent version of 'git' available in PATH.\nThe following error was reported by 'git':\n${stderr}`);
    const normalizePath = folderAN.startsWith('/')
        ? (p) => p.slice(1)
        : (p) => p;
    return stdout
        .replace(new RegExp(`(a|b)(${(0, escape_string_regexp_1.default)(`/${normalizePath(folderAN)}/`)})`, 'g'), '$1/')
        .replace(new RegExp(`(a|b)${(0, escape_string_regexp_1.default)(`/${normalizePath(folderBN)}/`)}`, 'g'), '$1/')
        .replace(new RegExp((0, escape_string_regexp_1.default)(`${folderAN}/`), 'g'), '')
        .replace(new RegExp((0, escape_string_regexp_1.default)(`${folderBN}/`), 'g'), '');
}
//# sourceMappingURL=patchCommit.js.map